﻿using CtrEditor.Convertidores;
using CtrEditor.Siemens;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace CtrEditor.ObjetosSim
{
    /// <summary>
    /// Interaction logic for ucBoton.xaml
    /// </summary>
    public class osBoton : osBase
    {
        // Otros datos y métodos relevantes para la simulación

        private string _nombre = "Boton";
        private float _tamano;
        private float _left;
        private float _top;
        private bool _estado;
        private string _tag;

        private Brush _color;
        private Brush _colorButton;       

        public Brush Color
        {
            get => _color;
            set
            {
                _color = value;
                OnPropertyChanged(nameof(Color));
            }
        }

        [Hidden]
        public Brush ColorButton
        {
            get => _colorButton;
            set
            {
                _colorButton = value;
                OnPropertyChanged(nameof(ColorButton));
            }
        }

        public override float Left
        {
            get => _left;
            set
            {
                _left = value;
                CanvasSetLeftinMeter(value);
                OnPropertyChanged(nameof(Left));
            }
        }
        public override float Top
        {
            get => _top;
            set
            {
                _top = value;
                CanvasSetTopinMeter(value);
                OnPropertyChanged(nameof(Top));
            }
        }
        public float Tamano
        {
            get => _tamano;
            set
            {
                _tamano = value;
                OnPropertyChanged(nameof(Tamano));
            }
        }
        public bool Estado
        {
            get => _estado;
            set
            {
                _estado = value;
                if (value)                     
                    ColorButton = Brushes.LightGreen;               
                else
                    ColorButton = Color;
                OnPropertyChanged(nameof(Estado));
            }
        }

        public override string Nombre
        {
            get => _nombre;
            set
            {
                if (_nombre != value)
                {
                    _nombre = value;
                    OnPropertyChanged(nameof(Nombre));
                }
            }
        }
        public string Tag
        {
            get => _tag;
            set
            {
                if (_tag != value)
                {
                    _tag = value;
                    OnPropertyChanged(nameof(Tag));
                }
            }
        }

        public void ButtonDownCommand()
        {
            Estado = true;
        }
        public void ButtonUpCommand()
        {
            Estado = false;
        }

        public osBoton()
        {
            Tamano = 0.30f;
            Estado = false;
            Tag = "M50.0";
            // Set initial color
            Color = Brushes.LightBlue;
        }

        public override void UpdateGeometryStart()
        {
            // Se llama antes de la simulacion

        }
        public override void UpdateGeometryStep()
        {
        }
        public override void UpdatePLC(PLCModel plc, int elapsedMilliseconds)
        {
            plc.EscribirTagBool(_tag, Estado);
        }

        public override void UpdateControl(int elapsedMilliseconds)
        {

        }
        public override void ucLoaded()
        {
            // El UserControl ya se ha cargado y podemos obtener las coordenadas para 
            // crear el objeto de simulacion
            ActualizarLeftTop();
        }
        public override void ucUnLoaded()
        {
            // El UserControl se esta eliminando
            // eliminar el objeto de simulacion
        }

    }

    public partial class ucBoton : UserControl, IDataContainer
    {
        public osBase? Datos { get; set; }

        public ucBoton()
        {
            InitializeComponent();
            this.DataContextChanged += OnDataContextChanged;
            this.Loaded += OnLoaded;
            this.Unloaded += OnUnloaded;
        }
        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucLoaded();
        }
        private void OnUnloaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucUnLoaded();
        }

        private void OnDataContextChanged(object sender, DependencyPropertyChangedEventArgs e)
        {
            Datos = e.NewValue as osBase;
        }

        private void Ellipse_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (Datos is osBoton osBotonData)
            {
                osBotonData.ButtonDownCommand();
                e.Handled = true; // Evita que el evento se propague
            }
        }

        private void Ellipse_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            if (Datos is osBoton osBotonData)
            {
                osBotonData.ButtonUpCommand();
                e.Handled = true; // Evita que el evento se propague
            }
        }

        public void Resize(float width, float height) { }
        public void Move(float LeftPixels, float TopPixels)
        {
            if (Datos != null)
            {
                Datos.Left = PixelToMeter.Instance.calc.PixelsToMeters(LeftPixels);
                Datos.Top = PixelToMeter.Instance.calc.PixelsToMeters(TopPixels);
            }
        }
        public void Rotate(float Angle) { }
        public void Highlight(bool State) { }
        public int ZIndex()
        {
            return 10;
        }
    }
}
