﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using CtrEditor.Convertidores;
using CtrEditor.Siemens;
using CtrEditor.Simulacion;
using FarseerPhysics.Dynamics;
using Microsoft.Xna.Framework;
using static System.Runtime.InteropServices.JavaScript.JSType;

namespace CtrEditor.ObjetosSim
{
    /// <summary>
    /// Interaction logic for ucGuia.xaml
    /// </summary>
    public class osGuia : osBase
    {
        private float _ancho;
        private float _altoGuia;
        private float _left;
        private float _top;
        private float _angulo;
        private string _nombre = "Guia";

        private simGuia Simulation_Guia;

        public override float Left
        {
            get => _left;
            set
            {
                _left = value;
                CanvasSetLeftinMeter(value);
                OnPropertyChanged(nameof(Left));
            }
        }
        public override float Top
        {
            get => _top;
            set
            {
                _top = value;
                CanvasSetTopinMeter(value);
                OnPropertyChanged(nameof(Top));
            }
        }

        public float Ancho
        {
            get => _ancho;
            set
            {
                _ancho = value;
                OnPropertyChanged(nameof(Ancho));
            }
        }
        public float AltoGuia
        {
            get => _altoGuia;
            set
            {
                _altoGuia = value;
                OnPropertyChanged(nameof(AltoGuia));
            }
        }

        public float Angulo
        {
            get => _angulo;
            set
            {
                _angulo = value;
                OnPropertyChanged(nameof(Angulo));
            }
        }

        public override string Nombre
        {
            get => _nombre;
            set
            {
                if (_nombre != value)
                {
                    _nombre = value;
                    OnPropertyChanged(nameof(Nombre));
                }
            }
        }

        private void ActualizarGeometrias()
        {
            if (_visualRepresentation is ucGuia uc)
                UpdateOrCreateLine(Simulation_Guia, uc.Guia);
        }

        public osGuia()
        {
            Ancho = 1;
            AltoGuia = 0.03f;
        }

        public override void UpdateGeometryStart()
        {
            // Se llama antes de la simulacion
            ActualizarGeometrias();
        }
        public override void UpdateControl(int elapsedMilliseconds)
        {
        }
        public override void UpdateGeometryStep()
        {
        }
        public override void UpdatePLC(PLCModel plc, int elapsedMilliseconds) { }
        public override void ucLoaded()
        {
            // El UserControl ya se ha cargado y podemos obtener las coordenadas para 
            // crear el objeto de simulacion
            ActualizarLeftTop();

            if (_visualRepresentation is ucGuia uc)
                Simulation_Guia = AddLine(simulationManager, uc.Guia);
        }
        public override void ucUnLoaded()
        {
            // El UserControl se esta eliminando
            // eliminar el objeto de simulacion
            simulationManager.Remove(Simulation_Guia);
        }


    }

    public partial class ucGuia : UserControl, IDataContainer
    {
        public osBase? Datos { get; set; }

        public ucGuia()
        {
            InitializeComponent();
            this.Loaded += OnLoaded;
            this.Unloaded += OnUnloaded;
        }
        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucLoaded();
        }
        private void OnUnloaded(object sender, RoutedEventArgs e)
        {
            Datos?.ucUnLoaded();
        }
        public void Resize(float width, float height)
        {
            if (Datos is osGuia datos)
                datos.Ancho = PixelToMeter.Instance.calc.PixelsToMeters(width);
        }
        public void Move(float LeftPixels, float TopPixels)
        {
            if (Datos != null)
            {
                Datos.Left = PixelToMeter.Instance.calc.PixelsToMeters(LeftPixels);
                Datos.Top = PixelToMeter.Instance.calc.PixelsToMeters(TopPixels);
            }
        }
        public void Rotate(float Angle)
        {
            if (Datos != null)
                if (Datos is osGuia datos)
                    datos.Angulo = Angle;
        }
        public void Highlight(bool State) { }
        public int ZIndex()
        {
            return 1;
        }

    }
}
